@@ -19,7 +19,7 @@
 # THE SOFTWARE.
 #
 
-from __future__ import absolute_import, unicode_literals
+from __future__ import absolute_import
 
 import re
 import base64
@@ -32,8 +32,8 @@ from .reply import Reply
 
 __all__ = ['ServerAuthError', 'AuthSession']
 
-noarg_pattern = re.compile(r'^([a-zA-Z0-9_-]+)$')
-witharg_pattern = re.compile(r'^([a-zA-Z0-9_-]+)\s+(.+)$')
+noarg_pattern = re.compile(br'^([a-zA-Z0-9_-]+)$')
+witharg_pattern = re.compile(br'^([a-zA-Z0-9_-]+)\s+(.+)$')
 
 
 class ServerAuthError(SmtpError):
@@ -51,6 +51,14 @@ class InvalidAuthString(ServerAuthError):
         super(InvalidAuthString, self).__init__(msg, reply)
 
 
+class InsecureMechanismError(ServerAuthError):
+
+    def __init__(self):
+        msg = 'Insecure authentication mechanism requires SSL'
+        reply = Reply('504', '5.5.4 '+msg)
+        super(InsecureMechanismError, self).__init__(msg, reply)
+
+
 class InvalidMechanismError(ServerAuthError):
 
     def __init__(self):
@@ -84,7 +92,7 @@ class AuthSession(object):
     def __str__(self):
         available = self.server_mechanisms
         if available:
-            return ' '.join(sorted([mech.name.decode() for mech in available]))
+            return ' '.join([mech.name.decode('ascii') for mech in available])
         else:
             raise ValueError('No mechanisms available')
 
@@ -99,8 +107,7 @@ class AuthSession(object):
 
     @property
     def server_mechanisms(self):
-        return [mech for mech in self.auth.server_mechanisms
-                if self.io.encrypted or not getattr(mech, 'insecure', False)]
+        return [mech for mech in self.auth.server_mechanisms]
 
     @property
     def client_mechanisms(self):
@@ -108,61 +115,59 @@ class AuthSession(object):
                 if self.io.encrypted or not getattr(mech, 'insecure', False)]
 
     def _server_challenge(self, challenge, response=None):
-        """
-        :type challenge: :py:obj:`str`
-        :type response: :py:obj:`str`
-        """
         if not response:
-            challenge_raw = base64.b64encode(challenge.encode()).decode()
+            challenge_raw = base64.b64encode(challenge).decode('ascii')
             Reply('334', challenge_raw).send(self.io, flush=True)
-            response = self.io.recv_line().decode()
-        if response == '*':
+            response = self.io.recv_line()
+        if response == b'*':
             raise AuthenticationCanceled()
         try:
-            return base64.b64decode(response).decode()
+            return base64.b64decode(response)
         except TypeError:
             raise InvalidAuthString()
 
     def server_attempt(self, arg):
         mechanism_name, mechanism_arg = self._parse_arg(arg)
-        for mechanism in self.server_mechanisms:
-            if mechanism.name.decode() == mechanism_name:
-                responses = []
-                while True:
-                    try:
-                        return mechanism.server_attempt(responses)
-                    except AuthenticationError as exc:
-                        raise UnexpectedAuthError(exc)
-                    except ServerChallenge as chal:
-                        if mechanism_arg:
-                            decoded_mechanism_arg = mechanism_arg
-                        else:
-                            decoded_mechanism_arg = None
-                        resp = self._server_challenge(chal.challenge.decode(),
-                                                      decoded_mechanism_arg)
-                        mechanism_arg = None
-                        chal.set_response(resp.encode())
-                        responses.append(chal)
+        mechanism = self.auth.get_server(mechanism_name)
+        if mechanism:
+            insecure = getattr(mechanism, 'insecure', False)
+            if insecure and not self.io.encrypted:
+                raise InsecureMechanismError()
+            responses = []
+            while True:
+                try:
+                    creds, _ = mechanism.server_attempt(responses)
+                    return creds
+                except AuthenticationError as exc:
+                    raise UnexpectedAuthError(exc)
+                except ServerChallenge as chal:
+                    resp = self._server_challenge(chal.challenge,
+                                                  mechanism_arg)
+                    mechanism_arg = None
+                    chal.set_response(resp)
+                    responses.append(chal)
         raise InvalidMechanismError()
 
     def _client_respond(self, mech, response, first=False):
         if first:
-            command = 'AUTH {0}'.format(mech.name.decode())
+            command = b' '.join((b'AUTH', mech.name))
             if response:
-                response_raw = base64.b64encode(response).decode('ascii')
-                command = ' '.join((command, response_raw))
+                response_raw = base64.b64encode(response)
+                command = b' '.join((command, response_raw))
         else:
-            command = base64.b64encode(response).decode('ascii')
+            command = base64.b64encode(response)
         self.io.send_command(command)
         self.io.flush_send()
-        ret = Reply(command='AUTH')
+        ret = Reply(command=b'AUTH')
         ret.recv(self.io)
         if ret.code == '334':
             return base64.b64decode(ret.message), ret
         return None, ret
 
     def client_attempt(self, authcid, secret, authzid, mech_name):
-        mechanism = self.auth.get(mech_name.encode())
+        if not mech_name:
+            raise InvalidMechanismError()
+        mechanism = self.auth.get_client(mech_name)
         if not mechanism:
             raise InvalidMechanismError()
         creds = AuthenticationCredentials(authcid, secret, authzid)
